@@ -39,7 +39,8 @@ class Tool:
                  max_input_bytes_per_file_parallel=FOUR_POINT_FIVE_GIGABYTES,
                  group_paired_ends=False, compress_inputs=False,
                  output_type=OutputType.FLAT_TEXT, expected_output_file_names=None,
-                 is_async=False, is_database_update=False, skip_decompression=False):
+                 is_async=False, is_database_update=False,
+                 skip_decompression=False, custom_docker_image_id=None):
         self.tool_name = tool_name
         self.tool_version = tool_version
         self.tool_args = tool_args
@@ -79,6 +80,7 @@ class Tool:
         self.is_async = is_async
         self.is_database_update = is_database_update
         self.skip_decompression = skip_decompression
+        self.custom_docker_image_id = custom_docker_image_id
         signal.signal(signal.SIGTERM, self._handle_termination)
         signal.signal(signal.SIGINT, self._handle_termination)
 
@@ -448,6 +450,7 @@ class Tool:
             # Note: multithreaded download may be broken with output_path refactor
             query_args = copy.deepcopy({
                 "custom_database_path": self.custom_database_path,
+                "custom_docker_image_id": self.custom_docker_image_id,
                 "database_name": self.database_name,
                 "database_version": self.database_version,
                 "input_files": input_files,
